# Gestion de queue avec Python
![category](développement)
![tag](python)

Présentation
------------

Python possède le module **queue** qui permet de gérer une file d'attente.

ce module est très pratique car il intègre sur la file:

* le mode de gestion (LIFO, FIFO, Priorité)
* la taille de la file
* l'ajout et la suppresion d'éléments dans cette file

les principales types de queues

* queue.Queue(maxsize=0)
* queue.LifoQueue(maxsize=0)
* queue.PriorityQueue(maxsize=0)

on peut pour chaque type de queue indiquer le nombre maximal d'élément

les principales fonctions

* queue.qsize()
* queue.empty()
* queue.full()
* queue.put(item[, block[, timeout]])
* queue.put_nowait(item)
* queue.get([block[, timeout]])
* queue.get_nowait()
* queue.task_done()
* queue.join()

Il faut noter que par défaut la fonction 

* get qu'un élément existe dans la file pour le donner
* put attend qu'une place dans la file se libère une place pour mettre l'élément

.. warning:

    cette attente peut être très longue, il est préférable d'ajouter un timeout exprimé en secondes

exemple simple d'utilisation simple

```python
import queue
q = queue.Queue()
for i in range(5):
    q.put(i)

while not q.empty():
    print q.get()
```

Utilisation
-----------

J'utilise très souvent le moduel *queue* avec des Threads pour paralléliser des traitements

UN exemple: le test d'un service web ... je souhaite lancer en parallèle 5 demandes avec un total
de 5000 demandes

```python
import time, random
import datetime
import requests
import json

import threading, Queue

class Test(threading.Thread):
    def __init__(self, q, ok, ko, end):
        threading.Thread.__init__(self)
        self.q = q
        self.ok = ok
        self.ko = ko
        self.end = end

    def run(self):
        while not self.q.empty():
            body = self.q.get()
            r = requests.post('http://test/ws', data=body)
            if r.status == 200:
                self.ok.put(body)
            else:
                self.ko.put(body)
        self.end.get()
        
parallel = 5
byqueue = 1000
end= queue.Queue()
ok= queue.Queue()
ko= queue.Queue()
queues = []
for i in range(1,parallel):
    q = queue.Queue()
    queues.append(q)
    end.put(1)
    for l in range(1, byqueue):
        q.put(json.dumps({"id": "%s-%s" % (len(queues), l)}))


print(datetime.datetime.now())
for q in queues:
    Test(q, ok, ko, end).start()

while not end.empty():
    time.sleep(1)

print(datetime.datetime.now())
print("total   : %s" % (parallel*byqueue))
print("error   : %s" % ko.qsize())
print("success : %s" % ok.qsize())
```
